@@ -794,7 +794,7 @@ midori_app_init (MidoriApp* app)
     app->instance = MidoriAppInstanceNull;
 
     #if HAVE_LIBNOTIFY
-    notify_init ("midori");
+    notify_init (PACKAGE_NAME);
     #else
     app->program_notify_send = g_find_program_in_path ("notify-send");
     #endif
@@ -1353,7 +1353,7 @@ midori_app_setup (gint               *argc,
     static GtkStockItem items[] =
     {
         { STOCK_IMAGE },
-        { STOCK_WEB_BROWSER },
+        { MIDORI_STOCK_WEB_BROWSER },
         { STOCK_NEWS_FEED },
         { STOCK_STYLE },
 
@@ -1364,7 +1364,7 @@ midori_app_setup (gint               *argc,
         { STOCK_SCRIPT,       N_("_Userscripts") },
         { STOCK_STYLE,        N_("User_styles") },
         { STOCK_TAB_NEW,      N_("New _Tab") },
-        { STOCK_TRANSFER,     N_("_Transfers"), GDK_CONTROL_MASK | GDK_SHIFT_MASK, GDK_KEY_J },
+        { MIDORI_STOCK_TRANSFER,     N_("_Transfers"), GDK_CONTROL_MASK | GDK_SHIFT_MASK, GDK_KEY_J },
         { MIDORI_STOCK_PLUGINS,      N_("Netscape p_lugins") },
         { STOCK_USER_TRASH,   N_("_Closed Tabs") },
         { STOCK_WINDOW_NEW,   N_("New _Window") },
@@ -1456,7 +1456,7 @@ void
 midori_error (const gchar* format,
               ...)
 {
-    g_printerr ("%s - ", _("Midori"));
+    g_printerr ("%s - ", g_get_application_name ());
     va_list args;
     va_start (args, format);
     g_vfprintf (stderr, format, args);
